/**
* Class GCode
* Contains a list of gcodecommands of type GCodeCOmmand
* Is able to visualize, print and save gcode.
* @author  Bas Pijls
*/
class GCode {
  String [] frontMatter;
  String [] backMatter;
  float zoomLevel = 1;
  float zRotation = 0;
  float xRotation = 0;
  PVector bbox1 = new PVector(), bbox0 = new PVector(Config.BEDSIZE, Config.BEDSIZE, Config.BEDSIZE);
  ArrayList<GCodeCommand> commands = new ArrayList<GCodeCommand>();
  int cursorIndex = 0;

  /**
  * Constructor loads intializer and finilizer gcode from files
  */
  GCode() {
    frontMatter = loadStrings("front_matter.gcode");
    backMatter = loadStrings("back_matter.gcode");
  }

  /**
  * Draws gcode to display. Rotate with mouse, zoom with up and down arrow
  */
  void draw() {
    translate(width/2, height/2);
    scale(2);
    pushMatrix(); 
    {
      scale(zoomLevel, zoomLevel, zoomLevel);
      zRotation = mouseX/(float)width*PI*2;
      xRotation = mouseY/(float)height*PI;
      
      if (keyPressed && keyCode == UP) zoomLevel += 0.01;
      if (keyPressed && keyCode == DOWN) zoomLevel -= 0.01;
            
      rotateX(xRotation);
      rotateZ(zRotation);
      translate(-Config.BEDSIZE/2, -Config.BEDSIZE/2);
      
      // Draw printbed
      fill(204, 204);
      strokeWeight(1);
      stroke(1);
      rect(0, 0, Config.BEDSIZE, Config.BEDSIZE);

      for (GCodeCommand command : commands) {
        command.draw();
      }

      cursorIndex = constrain(cursorIndex, 0, commands.size()-1);

      fill(0, 0, 255);
      GCodeCommand cursorCommand = commands.get(cursorIndex);
      pushMatrix();
      {
        translate(cursorCommand.targetPos.x, cursorCommand.targetPos.y, cursorCommand.targetPos.z);  
        sphere(2);
      }
      popMatrix();
    }
    popMatrix();
  }

  /**
  * Adds an new GCode command to the list of commands. 
  * Re-calculates the bounding box
  * Adds this GCode instance as parent to each command
  * @param GCodeCommand The object representing a gcode command
  */
  void addCommand(GCodeCommand command) {
    command.parentGCode = this;
    command.parentIndex = commands.size();
    commands.add(command);
  
    // finds the lower,left, front corner of the bounding box
    bbox0.x = command.targetPos.x < bbox0.x ? command.targetPos.x : bbox0.x; 
    bbox0.y = command.targetPos.y < bbox0.y ? command.targetPos.y : bbox0.y;
    bbox0.z = command.targetPos.z < bbox0.z ? command.targetPos.z : bbox0.z;
  
    // finds the upper, right, back corner of the bounding box
    bbox1.x = command.targetPos.x > bbox1.x ? command.targetPos.x : bbox1.x; 
    bbox1.y = command.targetPos.y > bbox1.y ? command.targetPos.y : bbox1.y;
    bbox1.z = command.targetPos.z > bbox1.z ? command.targetPos.z : bbox1.z;
  }

  /**
  * @return An array of strings, each containign a line of GCode
  */
  String [] generateGCodeStrings() {
    // Prepare an ArrayLis for the strings
    ArrayList<String> outputGCode = new ArrayList<String>();
    
    // Append the initializer gcode
    for (String s : frontMatter) outputGCode.add(s);
    // Append the command strings
    for (GCodeCommand command : commands)
      outputGCode.add(command.toString());
      // Append the finalizer gcode
    for (String s : backMatter) outputGCode.add(s);    
    
    // Convert the arraylist to an array of strings
    String [] outputStringArray = new String[outputGCode.size()];      
    outputStringArray = outputGCode.toArray(outputStringArray);
    return outputStringArray;
  }

  /**
  * @param fileName Filename of the GCode file to save
  */
  void saveGCode(String fileName) {
    saveStrings(fileName, generateGCodeStrings());
    println("File: " + fileName + " saved.");
  }
  
  void printGCode() {
    String [] gCodeStrings = generateGCodeStrings();
    for (String s : gCodeStrings) println(s);
  }
}

/**
* A class for a gcode command
*/
class GCodeCommand {
  String type;         // Command type (G0, M83, etc...)
  PVector targetPos;   // Position to travel to
  GCode parentGCode;   // List of GCdoe this command belongs to
  int parentIndex;     // Place in the parent list

  GCodeCommand(String type) {
    this.type = type;
  }

  String toString() { 
    return String.format("%s", type) ;
  }

  void draw() {
  }
}
