void generateHoppyCircle(PVector center, float radius, float hopHeight, int nHops, float nSteps, float zHeight, float flowRate) {

  center.z = zHeight;
  for (int iStep=0; iStep<nSteps; iStep++) {
    float t =  (float)iStep/nSteps;
    float fi = t * 2 * PI;

    float x = center.x + radius * cos(fi);
    float y = center.y + radius * sin(fi);
    float z = center.z;

    gcode.addCommand(new PrintCommand(new PVector(x, y, z), flowRate));
  }

  for (int iStep=0; iStep<nSteps; iStep++) {
    float t =  (float)iStep/nSteps;
    float fi = t * 2 * PI;

    float x = center.x + radius * cos(fi);
    float y = center.y + radius * sin(fi);
    float z = center.z;
    float hopZ = abs(sin((float)nHops/2 * 2* PI * t))*hopHeight;
    if (hopZ < 0.5) hopZ = 0;
    z += hopZ;

    gcode.addCommand(new PrintCommand(new PVector(x, y, z), flowRate));
  }
}

void generateHoppyLine(PVector startPos, PVector targetPos, float hopHeight, int nHops, float nSteps, float zHeight, float flowRate) {

  startPos.z = zHeight;
  targetPos.z = zHeight;

  gcode.addCommand(new TravelCommand(startPos));
  gcode.addCommand(new PrintCommand(targetPos, flowRate));

  PVector nextTarget;
  for (int iStep=0; iStep<nSteps; iStep++) {
    float t = iStep/(float)nSteps;
    nextTarget =  PVector.lerp(targetPos, startPos, t);    
    float hopZ = abs(sin((float)nHops/2 * 2* PI * t))*hopHeight;
    if (hopZ < 0.5) hopZ = 0;
    nextTarget.z += hopZ;
    gcode.addCommand(new PrintCommand(nextTarget, Config.FEEDRATE_PRINT, flowRate/2));
  }
}

void generateLine(PVector startPos, PVector targetPos, float zHeight, float flowRate) {
  startPos.z = zHeight;
  targetPos.z = zHeight;

  gcode.addCommand(new TravelCommand(startPos));
  gcode.addCommand(new PrintCommand(targetPos, flowRate));
}

void generateWobblyHelix(float centerX, float centerY, float radius0, float radius1, int nSteps, float frequency, float amplitude, float phase, float constrainMin, float constrainMax, boolean absolute, float zHeight, float flowRate) {

  float x, y, fi, radius, sineOffset;
  for (int iStep=0; iStep<nSteps; iStep++) {
    fi = (float)iStep/nSteps * 2 * PI;
    radius = map(iStep, 0, nSteps, radius0, radius1);
    sineOffset = sin(frequency * fi + phase);
    sineOffset = absolute ? abs(sineOffset) : sineOffset;
    sineOffset *= amplitude;
    sineOffset = constrain(sineOffset, constrainMin, constrainMax);
    radius += sineOffset;

    x = centerX + radius * cos(fi);
    y = centerY + radius * sin(fi);

    gcode.addCommand(new PrintCommand(new PVector(x, y, zHeight), flowRate));
  }
}


void generateHelix(float centerX, float centerY, float radius0, float radius1, float nTurns, int nSteps, float zHeightStart, float flowRate) {

  float x, y, fi, radius, z;
  float zHeightEnd = nTurns * Config.LAYER_HEIGHT*2;
  //int nSteps = (int)(nTurnSteps*nTurns);
  int nTurnSteps = (int)(nSteps/nTurns);  
  for (int iStep=0; iStep<nSteps; iStep++) {    
    fi = (float)iStep/nTurnSteps * 2 * PI;
    radius = map(iStep, 0, nSteps, radius0, radius1);
    float localFlowRate = map(radius, 0, 10, 0.02, 0.08);
    println(localFlowRate);
    x = centerX + radius * cos(fi);
    y = centerY + radius * sin(fi);
    z = map(iStep, 0, nSteps, zHeightStart, zHeightEnd);
    gcode.addCommand(new PrintCommand(new PVector(x, y, z), localFlowRate));
  }
}

void generateHoppyGrid(float left, float bottom, float right, float top, int nColumns, int nRows, float hopHeight, int nHops, int nSteps, float zHeight, float flowRate) {
  float x = left, y=bottom, xStart, xEnd, yStart, yEnd;

  for (int iY=0; iY<nRows; iY++) {
    y = map(iY, 0, nRows, bottom, top);

    xStart = left;
    xEnd = right;
    generateHoppyLine(new PVector(xEnd, y, zHeight), new PVector(xStart, y, zHeight), hopHeight, nHops, nSteps, zHeight, flowRate);
  }

  for (int iX=0; iX<nColumns; iX++) {
    x = map(iX, 0, nColumns, left, right);
    yStart = left;
    yEnd = right;
    generateHoppyLine(new PVector(x, yEnd, zHeight), new PVector(x, yStart, zHeight), hopHeight, nHops, nSteps, zHeight, flowRate);
  }
}


void generateGrid(float left, float bottom, float right, float top, int nColumns, int nRows, float zHeight, float flowRate) {   
  float x = left, y=bottom, xStart, xEnd, yStart, yEnd;
  PVector targetPos = new PVector(left, bottom, zHeight);

  gcode.addCommand(new TravelCommand(new PVector(left, bottom, zHeight)));
  gcode.addCommand(new PrintCommand(new PVector(right, bottom, zHeight), flowRate));
  gcode.addCommand(new PrintCommand(new PVector(right, top, zHeight), flowRate));
  gcode.addCommand(new PrintCommand(new PVector(left, top, zHeight), flowRate));
  gcode.addCommand(new PrintCommand(new PVector(left, bottom, zHeight), flowRate));

  for (int iY=0; iY<nRows; iY++) {
    y = map(iY, 0, nRows, bottom, top);

    if (iY % 2 == 1) {
      xStart = left; 
      xEnd = right;
    } else {
      xStart = right; 
      xEnd = left;
    }

    targetPos = new PVector(xEnd, y, zHeight);
    gcode.addCommand(new TravelCommand(targetPos));
    targetPos = new PVector(xStart, y, zHeight);
    gcode.addCommand(new PrintCommand(targetPos, flowRate));
  }

  for (int iX=0; iX<nColumns; iX++) {
    x = map(iX, 0, nColumns, left, right);

    if (iX % 2 == 1) {
      yStart = bottom; 
      yEnd = top;
    } else {
      yStart = top; 
      yEnd = bottom;
    }

    targetPos = new PVector(x, yEnd, zHeight);
    gcode.addCommand(new TravelCommand(targetPos));
    targetPos = new PVector(x, yStart, zHeight);
    gcode.addCommand(new PrintCommand(targetPos, flowRate));
  }
}
