class GCodeCommand {

    constructor(commandData) {     
        // if the command is a string, parse it
        // if the command is an object, use it as is
        
        if (typeof commandData === 'string') {
            commandData = this.parseString(commandData);
            return;
        }

        if (typeof commandData !== 'object') {
            throw new Error('Invalid command data');
        }

        this.command = commandData.command;
        this.description = commandData.description;
        this.parameters = commandData.parameters; // parameters should be an object { paramName: defaultValue }
    }

    parseString(commandString) {
        const commandParts = commandString.split(' ');
        const command = commandParts[0];
        const parameters = {};        
        for (let i = 1; i < commandParts.length; i++) {
            if (commandParts[i] === '') continue;
            const param = commandParts[i][0].toLowerCase();
            if (param === ';' || param === '(') {
                this.description = commandParts.slice(i+1).join(' ');
                break; // Stop parsing if it's a comment (semicolon) or a block comment (parentheses
            }
            const value = Number.parseFloat(commandParts[i].substr(1));
            parameters[param] = value;
            
        }
        this.command = command;
        this.parameters = parameters;
    }

    toHTML() {
        let inputsHTML = '';
        for (const [param, defaultValue] of Object.entries(this.parameters)) {
            inputsHTML += `${param.toUpperCase()}<input type="text" class="parameter-input w-12 p-1 border rounded" placeholder="${param.toUpperCase()}" value="${defaultValue}" data-param-name="${param}"> `;
        }
        
        return `
            <div class="p-2 mt-2 bg-gray-200 rounded shadow">
                <p class="text-gray-500 italic">${this.description}</p>
                <div data-command="${this.command}" data-description="${this.description}" class="command-line flex justify-between items-center">
                    
                    <span>${this.command} ${inputsHTML}</span>
                    
                    <button class="addCommand bg-blue-500 hover:bg-blue-700 text-white font-bold py-1 px-2 rounded">Add</button>
                    
                    
                </div>
            </div>
        `;
    }

    toString() {
        let commandString = this.command;
        for (const [param, value] of Object.entries(this.parameters)) {
            commandString += ` ${param.toUpperCase()}${value}`;
        }
        commandString += ` ; ${this.description}`;
        return commandString;
    }

}

let commandPalette = [
    { command: 'G0', description: 'Rapid move to a position', parameters: { x: '', y: '', z: '', f: ''} },
    { command: 'G1', description: 'Move to a position', parameters: { x: '', y: '', z: '', e: '', f: ''} },    
    { command: 'G28', description: 'Move to origin (home)', parameters: {} },
    { command: 'M82', description: 'Set extruder to absolute mode', parameters: {} },
    { command: 'M83', description: 'Set extruder to relative mode', parameters: {} },
    { command: 'G90', description: 'Set to absolute positioning', parameters: {} },
    { command: 'G91', description: 'Set to relative positioning', parameters: {} },
    { command: 'G92', description: 'Set position', parameters: { x: '', y: '', z: '', e: '' } },
    { command: 'M104', description: 'Set extruder temperature', parameters: { s: '200' } },
    { command: 'M109', description: 'Set extruder temperature and wait', parameters: { s: '200' } },
    { command: 'M140', description: 'Set bed temperature', parameters: { s: '60' } },
    { command: 'M190', description: 'Set bed temperature and wait', parameters: { s: '60' } },
    { command: 'M106', description: 'Fan on', parameters: { s: '' } },
    { command: 'M107', description: 'Fan off', parameters: {} },
    { command: 'M84', description: 'Stop idle hold', parameters: {} },
    { command: 'G20', description: 'Set units to inches', parameters: {} },
    { command: 'G21', description: 'Set units to millimeters', parameters: {} },
];
