let port;
let writer;
let reader;

function printMessage(message) {
    let consoleDiv = select('#printer-console');
    consoleDiv.html(`${message}<br>`, true);
}

async function connectPrinter() {
    if ('serial' in navigator) {
        try {            
            let baudrate = getBaudrate();
            port = await navigator.serial.requestPort();
            await port.open({ baudRate: baudrate });

            // Create a TextEncoder to send text to the writable stream
            const textEncoder = new TextEncoderStream();
            const writableStreamClosed = textEncoder.readable.pipeTo(port.writable);
            writer = textEncoder.writable.getWriter();

            // Prepare to read data
            const textDecoder = new TextDecoderStream();
            port.readable.pipeTo(textDecoder.writable);
            reader = textDecoder.readable.getReader();
            printMessage('Connected to the printer');

            waitForOk();

        } catch (err) {
            printMessage('Error connecting to the printer:', err);
        }
    } else {
        printMessage('Web Serial API not supported.');
    }
}

function isValidGCode(gcodeLine) {
    // Check if the line is not empty and does not start with a comment
    let notEmptyOrComment = gcodeLine.trim() && !gcodeLine.trim().startsWith(';');
    // use a regex to check if the line is an valid G- or M-command
    let validGCode = /^[gm]\d+/i.test(gcodeLine);
    return notEmptyOrComment && validGCode;
}

function stripGCodeComment(gcodeLine) {
    return gcodeLine.split(';')[0].trim();
}

async function sendGcodeLineByLine(gcode) {
    for (const line of gcode.split('\n')) {
        line = stripGCodeComment(line);
        printMessage(line);
        if (!isValidGCode(line)) {
            console.log(`Invalid G-code: ${line}`);
            continue;
        }

        await writer.write(line + '\n');

        waitForOk();        
    }
}

document.getElementById('connect-printer').addEventListener('click', connectPrinter);

document.getElementById('send-gcode').addEventListener('click', async () => {
    const gcode = document.getElementById('gcode-editor').value;
    if (port && writer && reader) {
        await sendGcodeLineByLine(gcode);
        printMessage('G-code sent successfully');
    } else {
        printMessage('Printer is not connected');
    }
});

async function disconnectPrinter() {
    if (reader) {
        await reader.cancel(); // Cancel ongoing read operations
        reader.releaseLock();
        reader = null;
    }
    if (writer) {
        await writer.close(); // Close the writer
        writer.releaseLock();
        writer = null;
    }
    if (port) {
        await port.close(); // Close the serial port
        port = null;
    }
    printMessage('Disconnected from the printer');
}

async function waitForOk(){
    let buffer = '';
    while (true) {
        const { value, done } = await reader.read();
        buffer += value;
        if (done) {
            printMessage('Reader stream closed');
            reader.releaseLock();
            break;
        }

        printMessage(`${value}`);

        if (buffer.includes('ok')) {
            break; 
        }
    }
}

document.getElementById('disconnect-printer').addEventListener('click', disconnectPrinter);
