// Blob class --------------------------------------------------------------------------------------------------------------
class Blob {

    constructor(s, xPosition, yPosition, xVelocity, yVelocity) {
        this.s = s;
        this.d = 10;
        this.xPosition = xPosition ?? s.width / 2;
        this.yPosition = yPosition ?? s.height / 2;
        this.xVelocity = xVelocity ?? s.random(-5, 5);
        this.yVelocity = yVelocity ?? s.random(-5, 5);
        this.gravity = 0.1;
        this.friction = 0.1;
    }

    update() {
        this.yVelocity += this.gravity;        
        this.xVelocity *= 1 - this.friction;
        this.yVelocity *= 1 - this.friction;        
        this.xPosition += this.xVelocity;
        this.yPosition += this.yVelocity;
    }

    bounce() {
        if (this.xPosition < 0 || this.xPosition > this.s.width) {
            this.xVelocity *= -1;
        }
        if (this.yPosition < 0 || this.yPosition > this.s.height) {
            this.yVelocity *= -1;
        }
    }

    draw() {
        this.s.circle(this.xPosition, this.yPosition, this.d);
    }
}


// Draw some shapes --------------------------------------------------------------------------------------------------------

const shapes = (s) => {
    s.setup = () => {
        s.createCanvas(400, 400);
    };

    s.draw = () => {
        s.background("#efefef");
        s.ellipse(100, 100, 100, 300);
        s.circle(400, 300, 40);
        s.rect(500, 400, 200, 150);
        s.line(30, 50, 400, 500);
    };
};

if (document.getElementById('shapesCanvas')) {
    let shapesCanvas = new p5(shapes, 'shapesCanvas');
}

// Draw some shapes

const colors = (s) => {
    s.setup = () => {
        s.createCanvas(400, 400);
    };

    s.draw = () => {
        // everything drawn after this function will be color (255,200, 10)
        s.fill(255, 200, 10);
        s.ellipse(100, 100, 100, 300);
        s.circle(400, 300, 40);

        // outline everything drawn after this function will be color (40,200, 100)
        s.stroke(40, 200, 100);
        s.rect(500, 400, 200, 150);

        // Everything is drawn with thick outlines
        s.strokeWeight(3);
        s.line(30, 50, 400, 500);

        // Reset appearance for the next frame
        s.fill(255);
        s.stroke(0);
        s.strokeWeight(1);
    };
};

if (document.getElementById('colorsCanvas')) {
    let colorsCanvas = new p5(colors, 'colorsCanvas');
}


// Bouncing ball ----------------------------------------------------------------------------------------------------------

const bounce = (s) => {

    let x = 0;
    let y = 0;
    let vx = 3;
    let vy = 3;
    let d = 20;

    s.setup = () => {
        s.createCanvas(400, 400);
        x = s.width / 2;
        y = s.height / 2;
        vx = -4;
        vy = 3;
    }

    s.draw = () => {
        s.background("#efefef");
        s.circle(x, y, d);
        x += vx;
        y += vy;
        if (x < d / 2 || x > s.width - d / 2) {
            vx *= -1;
        }
        if (y < d / 2 || y > s.height - d / 2) {
            vy *= -1;
        }
    }
}

if (document.getElementById('bounceCanvas')) {
    let bounceCanvas = new p5(bounce, 'bounceCanvas');
}

// Spawn 100 blobs on the screen ------------------------------------------------------------------------------------------

const lotsOfBlobs = (s) => {

    let blobs = [];

    s.setup = () => {
        s.createCanvas(400, 400);
        for (let i = 0; i < 100; i++) {
            blobs.push(new Blob(s));
        }
    }

    s.draw = () => {
        s.background("#efefef");
        for (let i = 0; i < blobs.length; i++) {
            blobs[i].update();
            blobs[i].draw();
        }
    }
}

if (document.getElementById('lotsOfBlobsCanvas')) {
    let lotsOfBlobsCanvas = new p5(lotsOfBlobs, 'lotsOfBlobsCanvas');
}

// Spawn blobs on mouse click

const spawningBlobs = (s) => {

    let blobs = [];

    s.setup = () => {
        s.createCanvas(400, 400);
    }

    s.draw = () => {
        s.background("#efefef");
        for (let i = 0; i < blobs.length; i++) {
            blobs[i].update();
            blobs[i].draw();
        }

        if (s.mouseIsPressed) {
            blobs.push(new Blob(s, s.mouseX, s.mouseY));
        }
    }
}

if (document.getElementById('spawningBlobsCanvas')) {
    let lotsOfBlobsCanvas = new p5(spawningBlobs, 'spawningBlobsCanvas');
}


// -----------------------------------------------------------------------------------------------------------
// Animate a circle moving across the screen and make the framerate adjustable with a slider

const animatedCircle = (s) => {

    let x = 0;
    let y = 0;
    let vx = 20;
    let vy = 10;
    let d = 100;
    let slider;

    s.setup = () => {
        s.createCanvas(400, 400);
        ui = s.select('#animatedCircleUI');
        x = s.width / 2;
        y = s.height / 2;
        s.createSpan('<br>framerate:').parent(ui);
        slider = s.createSlider(1, 60, 30, 1).parent(ui);
        //s.blendMode(s.ADD);
        s.noStroke();
        s.background(0);
    }

    s.draw = () => {

        s.fill(0, 0, 0, 5);
        s.rect(0, 0, s.width, s.height);

        if (s.frameCount % (61 - slider.value()) == 0) {
            s.fill(255, 255, 255);
            s.circle(x, y, d);
            x += vx;
            //y += vy;
            if (x < d / 2 || x > s.width - d / 2) {
                vx *= -1;
            }
            if (y < d / 2 || y > s.height - d / 2) {
                vy *= -1;
            }
        }
    }
}

if (document.getElementById('animatedCircleCanvas')) {
    let animatedCircleCanvas = new p5(animatedCircle, 'animatedCircleCanvas');
}

// ------------------------------------------------------------------------------------------------------------------------------------
// Physics
// ------------------------------------------------------------------------------------------------------------------------------------

const physics = (s) => {

    let blobs = [];
    let gravitySlider;
    let frictionSlider;
    let clearCheckbox;
    
    s.setup = () => {
        s.createCanvas(400, 400);
        s.noStroke();
        s.background(0,0,0,3);
        blob = new Blob(s, s.width / 2, s.height / 4);
        blobs.push(blob);

        // slider for gravity
        s.createSpan('<br>Gravity').parent('physicsUI');
        gravitySlider = s.createSlider(0, 1, 0.1, 0.01).parent('physicsUI');
        // slider for friction
        s.createSpan('<br>Friction').parent('physicsUI');
        frictionSlider = s.createSlider(0, 0.05, 0.03, 0.001).parent('physicsUI');
        // checkbox for clear background
        clearCheckbox = s.createCheckbox('Clear background', true).parent('physicsUI');

    };

    s.draw = () => {
        if (clearCheckbox.checked()) s.background(0,0,0,3);

        for (let blob of blobs) {
            blob.gravity = gravitySlider.value();
            blob.friction = frictionSlider.value();
            blob.update();
            blob.bounce();
            blob.draw();
        }

        if (s.mouseIsPressed) {
            let blob = new Blob(s, s.mouseX, s.mouseY);
            blobs.push(blob);
        }
    };
}

if (document.getElementById('physicsCanvas')) {
    let physicsCanvas = new p5(physics, 'physicsCanvas');
}


function initSerialPort(s, uiID, baudRate) {
    s.createCanvas(400, 400);

    let port = s.createSerial();

    
        // add a button
        s.createSpan('<br>').parent(uiID);
        let openButton = s.createButton('open serial port').parent(uiID);
        openButton.mousePressed(() => {            
            port.open(baudRate);
        });
        s.createSpan('<br>').parent(uiID);

        let closeButton = s.createButton('close serial port').parent(uiID);
        closeButton.mousePressed(() => {
            port.close();
        });
        s.createSpan('<br>').parent(uiID);
    
    return port;
}

// ------------------------------------------------------------------------------------------------------------------------------------
// Serial
// ------------------------------------------------------------------------------------------------------------------------------------
const serial = (s) => {

    let port;

    s.setup = () => {
        s.createCanvas(400, 400);
        port = initSerialPort(s, 'serialportUI', 9600);
    }

    s.draw = () => {
        s.background(0);
        if (port.available() > 0) {
            let data = port.readUntil('\n');
            if (data !== null) {
                let sensorValue = Number(data);
                s.fill(255);
                s.text(data, 10, 10);

                let shade = s.map(sensorValue, 0, 1023, 50, 255);
                let circleSize = s.map(sensorValue, 0, 1023, 100, 400);

                s.fill(shade);
                s.circle(s.width / 2, s.height / 2, circleSize);
            }
        }
    }
}

if (document.getElementById('serialportCanvas')) {
    let serialportCanvas = new p5(serial, 'serialportCanvas');
}

// ------------------------------------------------------------------------------------------------------------------------------------
// color picker
// ------------------------------------------------------------------------------------------------------------------------------------
const colorPicker = (s) => {

    let port;
    let brightnessSlider;

    s.setup = () => {
        s.createCanvas(400, 400);

        port = initSerialPort(s, 'colorPickerUI', 9600);
        
        // brightness slider
        s.createSpan('<br>brightness: ').parent('colorPickerUI');
        brightnessSlider = s.createSlider(0, 255, 100).parent('colorPickerUI');

        s.noStroke();
        s.colorMode(s.HSB, 255);
        s.background(0);            
    }

    s.draw = () => {
        s.background(0);

        s.colorMode(s.HSB, 255);
        let w = s.width/255;
        let h = s.height/255;
        for (let iColumn=0; iColumn<255; iColumn+=4){
            for (let iRow=0; iRow<255; iRow+=4){
                s.fill(iColumn, iRow, brightnessSlider.value());
                s.rect(iColumn*w, iRow*h, w*5, h*5);
            }
        }

        // read the color from the canvas        
        if (s.mouseIsPressed){
            s.colorMode(s.RGB, 255);        
            let pickedColor = s.get(s.mouseX, s.mouseY);
            s.fill(pickedColor);
            s.circle(s.mouseX, s.mouseY, 100);
            let redByte = s.red(pickedColor);
            let greenByte = s.green(pickedColor);
            let blueByte = s.blue(pickedColor);
            let packet = [redByte, greenByte, blueByte, 101];
            if (port.opened()) 
                port.write(packet);    
        }

    }
}

if (document.getElementById('colorPickerCanvas')) {
    let colorPickerCanvas = new p5(colorPicker, 'colorPickerCanvas');
}


// ------------------------------------------------------------------------------------------------------------------------------------
// Websocket
// ------------------------------------------------------------------------------------------------------------------------------------
WebSocketDemo = function (s) {

    var socket;
    let message = "";
    let pickedColor;

    s.setup = function () {
        s.createCanvas(400, 400);
        s.fill(255);
        // add a button to open the websocket
        s.createSpan('<br>').parent('websocketUI');
        let socketServerInput = s.createInput('socket server').parent('websocketUI');

        s.createSpan('<br>').parent('websocketUI');
        let openButton = s.createButton('open websocket').parent('websocketUI');

        // brightness slider
        s.createSpan('<br>brightness: ').parent('websocketUI');
        brightnessSlider = s.createSlider(0, 255, 100).parent('websocketUI');

        s.noStroke();
        s.colorMode(s.HSB, 255);
        s.background(0);

        openButton.mousePressed(() => {
            socket = new WebSocket(socketServerInput.value());

            socket.onopen = () => {
                socket.send('Hello!');
                message = 'Client notified socket has opened';
            };
            socket.onclose = function (event) {
                message = 'Client notified socket has closed';
            };
            socket.onmessage = function (event) {
                message = 'Client received a message: ' + event.data;
            }
        });

        // add a button to close the websocket
        s.createSpan('<br>').parent('websocketUI');
        let closeButton = s.createButton('close websocket').parent('websocketUI');
        closeButton.mousePressed(() => {
            socket.close();
        });

    };

    s.draw = function () {
        s.background(0);
        s.text(message, 50, 50);

        s.background(0);

        s.colorMode(s.HSB, 255);
        let w = s.width / 255;
        let h = s.height / 255;
        for (let iColumn = 0; iColumn < 255; iColumn += 4) {
            for (let iRow = 0; iRow < 255; iRow += 4) {
                s.fill(iColumn, iRow, brightnessSlider.value());
                s.rect(iColumn * w, iRow * h, w * 5, h * 5);
            }
        }

        // read the color from the canvas
        // if mouse is pressed and moude position is within the canvas
        if (s.mouseIsPressed && s.mouseX < s.width && s.mouseY < s.height) {
            s.colorMode(s.RGB, 255);
            pickedColor = s.get(s.mouseX, s.mouseY);
            s.fill(pickedColor);
            s.circle(s.mouseX, s.mouseY, 100);

        }
    }

    s.mouseReleased = function () {
        let redByte = s.red(pickedColor);
        let greenByte = s.green(pickedColor);
        let blueByte = s.blue(pickedColor);
        let packet = [redByte, greenByte, blueByte, 101];
        if (socket.readyState !== WebSocket.CLOSED) {
            socket.send(packet);
        }
    }

};

if (document.getElementById('websocketCanvas')) {
    let websocketCanvas = new p5(WebSocketDemo, 'websocketCanvas');
}


// ------------------------------------------------------------------------------------------------------------------------------------
// Spring
// ------------------------------------------------------------------------------------------------------------------------------------

const spring = (s) => {
    let x;
    let vx = 0;
    let ax = 0;
    let mass = 24;
    let k = 0.1;
    let damping = 0.9;

    let massSlider;
    let kSlider;
    let dampingSlider;

    s.setup = () => {
        s.createCanvas(400, 400);
        s.noStroke();
        s.background(0);
        x = s.width / 2;

        // sldier for mass
        s.createSpan('<br>Mass').parent('springUI');
        massSlider = s.createSlider(0, 100, 24, 1).parent('springUI');
        // slider for spring constant
        s.createSpan('<br>Spring constant').parent('springUI');
        kSlider = s.createSlider(0.0, 10, 0.1, 0.01).parent('springUI');
        // slider for damping
        s.createSpan('<br>Damping').parent('springUI');
        dampingSlider = s.createSlider(0.0, 10, 0.9, 0.01).parent('springUI');
    }

    s.draw = () => {
        s.background(0);

        // stretch spring if mouse is pressed and mouse position is in canvas
        if (s.mouseIsPressed && s.mouseX > 0 && s.mouseX < s.width) {
            x = s.mouseX;
            vx = 0;
        }
        mass = massSlider.value();
        k = kSlider.value();
        damping = dampingSlider.value();

        let springForce = k * (x - s.width / 2) + (vx * damping);
        ax = -springForce / mass;
        vx += ax;
        x += vx;

        s.fill(255);
        s.ellipse(x, s.height / 2, 50, 50);
        s.line(x, s.height / 2, s.width / 2, s.height / 2);

        s.text(`Mass: ${mass}`, 10, 20);
        s.text(`Spring constant: ${k}`, 10, 40);
        s.text(`Damping: ${damping}`, 10, 60);

    }
}

if (document.getElementById('springCanvas')) {
    let springCanvas = new p5(spring, 'springCanvas');
}
