class Blob {

    constructor(s, xPosition, yPosition, xVelocity, yVelocity) {
        this.s = s;
        this.d = 10;
        this.xPosition = xPosition ?? s.width / 2;
        this.yPosition = yPosition ?? s.height / 2;
        this.xVelocity = xVelocity ?? s.random(-5, 5);
        this.yVelocity = yVelocity ?? s.random(-5, 5);
        this.gravity = 0.1;
        this.friction = 0.1;
    }

    update() {
        this.yVelocity += this.gravity;        
        this.xVelocity *= 1 - this.friction;
        this.yVelocity *= 1 - this.friction;        
        this.xPosition += this.xVelocity;
        this.yPosition += this.yVelocity;
    }

    bounce() {
        if (this.xPosition < 0 || this.xPosition > this.s.width) {
            this.xVelocity *= -1;
        }
        if (this.yPosition < 0 || this.yPosition > this.s.height) {
            this.yVelocity *= -1;
        }
    }

    draw() {
        this.s.circle(this.xPosition, this.yPosition, this.d);
    }
}



// spring using hookes law

const spring = (s) => {

    let x;
    let vx = 0;
    let ax = 0;
    let mass = 24;
    let k = 0.1;
    let damping = 0.9;

    let massSlider;
    let kSlider;
    let dampingSlider;

    
    s.setup = () => {
        s.createCanvas(400, 400);
        s.noStroke();
        s.background(0);
        x = s.width / 2;

        // sldier for mass
        s.createSpan('<br>Mass').parent('springUI');
        massSlider = s.createSlider(0, 100, 24, 1).parent('springUI');
        // slider for spring constant
        s.createSpan('<br>Spring constant').parent('springUI');
        kSlider = s.createSlider(0.0, 10, 0.1, 0.01).parent('springUI');
        // slider for damping
        s.createSpan('<br>Damping').parent('springUI');
        dampingSlider = s.createSlider(0.0, 10, 0.9, 0.01).parent('springUI');
    }

    s.draw = () => {
        s.background(0);

        // stretch spring if mouse is pressed and mouse position is in canvas
        if (s.mouseIsPressed && s.mouseX > 0 && s.mouseX < s.width) {
            x = s.mouseX;
            vx = 0;
        }
        mass = massSlider.value();
        k = kSlider.value();
        damping = dampingSlider.value();

        let springForce = k * (x - s.width / 2) + (vx * damping);
        ax = -springForce / mass;
        vx += ax;
        x += vx;

        s.fill(255);
        s.ellipse(x, s.height / 2, 50, 50);
        s.line(x, s.height / 2, s.width / 2, s.height / 2);

        s.text(`Mass: ${mass}`, 10, 20);
        s.text(`Spring constant: ${k}`, 10, 40);
        s.text(`Damping: ${damping}`, 10, 60);

    }

}

if (document.getElementById('springCanvas')) {
    let springCanvas = new p5(spring, 'springCanvas');
}

// smoke particles using the blob class

const smoke = (s) => {

    let blobs = [];
    let gravitySlider;
    let frictionSlider;
    let nParticlesSlider;
    let clearCheckbox;
    let opacitySlider;
    let sizeSlider;
   
    s.setup = () => {
        s.createCanvas(400, 400);
        s.noStroke();
        s.background(0);
        blob = new Blob(s, s.width / 2, s.height / 4);
        blobs.push(blob);
        s.blendMode(s.ADD);

        // slider for gravity
        s.createSpan('<br>Gravity ').parent('smokeUI');
        gravitySlider = s.createSlider(-1, 0, -0.1, 0.01).parent('smokeUI');
        // slider for friction
        s.createSpan('<br>Friction ').parent('smokeUI');
        frictionSlider = s.createSlider(0, 0.05, 0, 0.001).parent('smokeUI');
        // slider for the number of particles
        s.createSpan('<br>Spawnrate ').parent('smokeUI');
        nParticlesSlider = s.createSlider(0, 100, 50, 1).parent('smokeUI');
        // slider for the opacity
        s.createSpan('<br>Opacity ').parent('smokeUI');
        opacitySlider = s.createSlider(0, 20, 10, 1).parent('smokeUI');
        // sl;ider for the size of the particles
        s.createSpan('<br>Size ').parent('smokeUI');
        sizeSlider = s.createSlider(0, 20, 10, 1).parent('smokeUI');
        // slider for the spread of the particles
        s.createSpan('<br>Spread ').parent('smokeUI');
        spreadSlider = s.createSlider(0, 20, 10, 1).parent('smokeUI');

        // checkbox for clear background
        clearCheckbox = s.createCheckbox('Clear background', true).parent('smokeUI');
    };

    s.draw = () => {

        if (clearCheckbox.checked()) {
            s.clear();
            s.background(0);
        }

        for (let blob of blobs) {
            blob.gravity = gravitySlider.value();
            blob.friction = frictionSlider.value();

            s.fill(255, 255, 255, opacitySlider.value());
            blob.update();
            blob.draw();
        }

        let nParticles = nParticlesSlider.value();

        let cx = s.width / 2;
        let cy = s.height * 0.75;

        // if mouse is pressed and on canvas, spawn particles at mouse position
        if (s.mouseIsPressed && s.mouseX > 0 && s.mouseX < s.width && s.mouseY > 0 && s.mouseY < s.height) {
            cx = s.mouseX;
            cy = s.mouseY;
        }

        for (let i = 0; i < nParticles; i++) {
            let spread = spreadSlider.value();
            let blob = new Blob(s, cx, cy, s.random(-spread, spread), s.random(-spread, -spread));
            let randomSize = sizeSlider.value();
            blob.d = s.random(randomSize + 10, randomSize + 40);
            blobs.push(blob);
        }

        // use filter to remove particles that are off screen
        blobs = blobs.filter(blob => blob.xPosition > 0 && blob.xPosition < s.width && blob.yPosition > 0 && blob.yPosition < s.height);

        s.fill(255);
        s.text(`Particles: ${blobs.length}`, 10, 20);
        s.text(`Gravity: ${gravitySlider.value()}`, 10, 40);
        s.text(`Friction: ${frictionSlider.value()}`, 10, 60);
        s.text(`Spawnrate: ${nParticlesSlider.value()}`, 10, 80);
        s.text(`Opacity: ${opacitySlider.value()}`, 10, 100);
    };
}

if (document.getElementById('smokeCanvas')) {
    let smokeCanvas = new p5(smoke, 'smokeCanvas');
}