#ifndef _KEYPAD_HPP
#define _KEYPAD_HPP

#include "config.hpp"
#include <Arduino.h>
#include <Wire.h>

#define createColor(r,g,b) (uint32_t)r<<16 | (uint32_t)g<<8 | (uint32_t)b

#define red(a) (a>>16)&0xFF
#define green(a) (a>>8)&0xFF
#define blue(a) a&0xFF

typedef struct
{
    uint8_t buttons = 0;
    uint16_t sensor1 = 0;
    uint16_t sensor2 = 0;
    uint8_t knobButton = 0;
    uint16_t knobPosition = 0;
} InputState;

class KeyPad
{

public:
    uint8_t *buttonOrder;
    uint8_t i2cAddress = 0;
    uint8_t lastPollButtonStates = 0;
    uint8_t buttonsRisingEdge = 0;
    uint8_t buttonsFallingEdge = 0;
    uint32_t ledColorStates[N_KEYPAD_KEYS];
    InputState inputState;
    uint8_t ledsChanged = true;
    uint8_t toneStates[128];

    KeyPad(uint8_t i2cAddress, uint8_t buttonOrder[])
    {
        this->i2cAddress = i2cAddress;
        this->buttonOrder = buttonOrder;
    }

    void update()
    {
        lastPollButtonStates = inputState.buttons;

        Wire.requestFrom(i2cAddress, 8); 
        while (Wire.available())
        { 
            inputState.buttons = Wire.read();
            inputState.sensor1 = 1024 - (Wire.read() + Wire.read() * 256);
            inputState.sensor2 = 1024 - (Wire.read() + Wire.read() * 256);
            inputState.knobButton = Wire.read();
            inputState.knobPosition = (Wire.read() + Wire.read() * 256) / 2;
        }

        buttonsFallingEdge = ~inputState.buttons & lastPollButtonStates;
        buttonsRisingEdge = ~lastPollButtonStates & inputState.buttons;
    }

    bool getButtonDown(uint8_t buttonIndex)
    {
        return (inputState.buttons & (1 << buttonOrder[buttonIndex]));
    }

    bool getButtonPressed(uint8_t buttonIndex)
    {
        return buttonsRisingEdge & (1 << buttonOrder[buttonIndex]);
    }

    bool getButtonReleased(uint8_t buttonIndex)
    {
        return buttonsFallingEdge & (1 << buttonOrder[buttonIndex]);
    }

    void setLEDColor(uint8_t iButton, uint8_t red, uint8_t green, uint8_t blue)
    {
        ledColorStates[iButton] = ((uint32_t)red << 16) |
                                  ((uint32_t)green << 8) |
                                  (uint32_t)blue;

        Wire.beginTransmission(i2cAddress);

        Wire.write('l');
        Wire.write(iButton);
        Wire.write(red);
        Wire.write(blue);
        Wire.write(green);

        Wire.endTransmission(i2cAddress);
    }

    void clearLEDs()
    {
        for (uint8_t iLed = 0; iLed < N_KEYPAD_KEYS; iLed++)
        {
            setLEDColor(iLed, 0, 0, 0);
        }
    }

    String toString()
    {

        String stateString = "\nRaw inputState: ";

        for (int iByte = 0; iByte < 8; iByte++)
        {
            stateString += String(((uint8_t *)&inputState)[iByte], HEX) + " ";
        }

        stateString = "\nColors: ";

        for (int iColor = 0; iColor < 8; iColor++)
        {
            stateString += String(ledColorStates[iColor], HEX) + " ";
        }

        stateString += "\nButtons: ";
        for (uint8_t iButton = 0; iButton < N_KEYPAD_KEYS; iButton++)
        {
            stateString += String(getButtonDown(iButton)) + " ";
        }
        stateString += "\n Sensor values: ";
        stateString += String(inputState.sensor1) + " " + String(inputState.sensor2);

        stateString += "\nEncoder position: " + String(inputState.knobPosition);
        stateString += "\nEncoder button: " + String(inputState.knobButton);
        stateString += "\n";
        return stateString;
    }
};

#endif