import bpy
import mathutils
# ExportHelper is a helper class, defines filename and
# invoke() function which calls the file selector.
from bpy_extras.io_utils import ExportHelper
from bpy.props import StringProperty, BoolProperty, EnumProperty
from bpy.types import Operator


def write_data(context, filepath, data):
    print("Writing data")
    f = open(filepath, 'w', encoding='utf-8')
    f.write(data)
    f.close()
    return {'FINISHED'}
        
# creates a list of all connected vertices per vertex.
# the first index in each entry is the vertex index itself, subsequent indices are indeces of
# connected vertices
def get_connected_vertices(mesh):
    
    nVertices = len(mesh.vertices)    
    # Preallocate an list of lists, already filled with a vertex index
    connectedVertices = [[iVertex] for iVertex in range(nVertices)]
        
    # for all the edges
    for edge in mesh.edges:
        connectedVertices[edge.vertices[0]].append(edge.index)
        connectedVertices[edge.vertices[0]].append(edge.vertices[1])
        connectedVertices[edge.vertices[1]].append(edge.index)
        connectedVertices[edge.vertices[1]].append(edge.vertices[0])    

    return connectedVertices

#creates array containing:
# first 3 elements are the normal for each pivot vertex
# subsequent elements are x, y and z for each connected vertex w.r.t. pivot vertex
def create_vertex_array(mesh, vertexIndices):
    
    firstVertex = mesh.vertices[vertexIndices[0]]        
    vertexArray = [firstVertex.co, firstVertex.normal] #store position and normal
    
    averageOutDir = mathutils.Vector((0.0, 0.0, 0.0))
    
    # for each subsequent vertex index
    for iVertexIndex in range(1, len(vertexIndices),2):
        iEdge = vertexIndices[iVertexIndex]
        iVertex = vertexIndices[iVertexIndex+1] #find the vertex location for this vertex
        vertexCo = mesh.vertices[iVertex].co #set first vector to be vertex position
        outDir = (vertexCo - firstVertex.co).normalized() #calculate the normalized outgoing direction w.r.t. the pivot vertex
        vertexArray.append(iEdge)
        vertexArray.append(outDir) # add the direction vector components to the list
        averageOutDir += outDir
    
    # calculate the average direction of all outgoing vertices
    averageOutDir /= len(vertexIndices)-1
    averageOutDir.normalize()
    # replace vertex normal with average output direction
    # Choose best strategy, comment line if other strategy works better
    vertexArray[1] = averageOutDir
    
    return vertexArray

def getLength(vertexIndex):
    return vertexIndex[2]

# Compose an array of edge data. Each element contains a list of [vertexIndex0, vertexIndex1, edgeLength]
def create_edge_array(mesh):
    vertexIndex = list(map(lambda e: list(e.vertices), mesh.edges))
    for iEdge in range(0, len(vertexIndex)):
        edgeLength = (mesh.vertices[vertexIndex[iEdge][0]].co - mesh.vertices[vertexIndex[iEdge][1]].co).length
        vertexIndex[iEdge].append(edgeLength)
        
    vertexIndex.sort(key=getLength)        
    return vertexIndex;

# exports openscad file
def export_connected_vertices(context, filePath):
    
    # get selected mesh
    mesh = bpy.context.selected_objects[0].to_mesh()
    
    connectedVertices = get_connected_vertices(mesh)
    vertexData = []
    for vertexIndices in connectedVertices:
        vertexArray = create_vertex_array(mesh, vertexIndices)
        vertexData.append(vertexArray)
    
    #vertexIndexString = f'vertexIndexData={str(connectedVertices)}\n'
            
    #convert vector string data to openscad vectors
    vertexDataString = str(vertexData).replace("Vector",'').replace("((","[").replace("))","]").replace(']],',']],\n')
    
    #convert edge vertex indices to string
    edgeDataString = create_edge_array(mesh)
    
    #add openscad variable names to string
    outputString = f'vertexData = {vertexDataString};\n\n edgeData = {edgeDataString};\n\n'
    
    print(outputString)
    #write data to selected output file
    write_data(context, filePath, outputString)
    
    return {'FINISHED'}

class ExportPFKData(Operator, ExportHelper):
    """Generate vertex and edge info to be used by OpenSCAD to generate 2d vertices and edges"""
    bl_idname = "export_test.some_data"  # important since its how bpy.ops.import_test.some_data is constructed
    bl_label = "Export PFK Data"

    # ExportHelper mixin class uses this
    filename_ext = ".scad"

    filter_glob: StringProperty(
        default="*.scad",
        options={'HIDDEN'},
        maxlen=255,  # Max internal buffer length, longer would be clamped.
    )

    def execute(self, context):
        return export_connected_vertices(context, self.filepath)


# Only needed if you want to add into a dynamic menu
def menu_func_export(self, context):
    self.layout.operator(ExportPFKData.bl_idname, text="Openscad PFK Data")


def register():
    bpy.utils.register_class(ExportPFKData)
    bpy.types.TOPBAR_MT_file_export.append(menu_func_export)


def unregister():
    bpy.utils.unregister_class(ExportPFKData)
    bpy.types.TOPBAR_MT_file_export.remove(menu_func_export)


if __name__ == "__main__":
    register()

    # test call
    #bpy.ops.export_test.some_data('INVOKE_DEFAULT')
