// Represents a 2D vector toolpath
class ToolPath {

  ArrayList<PVector> vertices;
  ArrayList<PVector> angles = new ArrayList<PVector>();
  ArrayList<PVector> dAngles = new ArrayList<PVector>();

  float angle;
  int activeCoordinateIndex = 0;

  PVector bbMin = new PVector(), bbMax = new PVector(), center = new PVector();
  int iCursor;

  ToolPath() {
    setVertices(new ArrayList<PVector>());
  }

  ToolPath(ArrayList<PVector> vertices) {
    setVertices(vertices);
  }

  void getBBox() {
    bbMin = vertices.get(0).copy();
    bbMax = vertices.get(0).copy();

    for (PVector vertex : vertices) {
      if (vertex.x < bbMin.x) bbMin.x = vertex.x;
      if (vertex.x > bbMax.x) bbMax.x = vertex.x;
      if (vertex.y < bbMin.y) bbMin.y = vertex.y;
      if (vertex.y > bbMax.y) bbMax.y = vertex.y;
    }

    bbMin.z = 0;
    bbMax.z = 0;
  }


  // subtract center from all toolpath vertices
  void moveToCenter() {
    getBBox();
    center = PVector.div(PVector.add(bbMin, bbMax), 2.0);
    for (PVector v : vertices) {
      v.sub(center);
    }
    getBBox();
    // Calculate new centrer
    center = PVector.div(PVector.add(bbMin, bbMax), 2.0);
  }

  // Based on t (0<=t<=1.0)
  PVector getInterpolatedVertex(float t) {
    float floatingIndex = map(t, 0, 1, 0, vertices.size()-1); 
    float tLerp = floatingIndex - floor(floatingIndex);  

    PVector coordinate0 = vertices.get(floor(floatingIndex));
    PVector coordinate1 = vertices.get(ceil(floatingIndex));    
    PVector lerpVert =  PVector.lerp(coordinate0, coordinate1, tLerp);
    if (coordinate1.z > 0) {
      lerpVert.z = 1.0;
    }
    return lerpVert;
  }

  // Set the vertices and recalculate the bounding box, center and angles.
  void setVertices(ArrayList<PVector> vertices) {
    this.vertices = vertices;

    moveToCenter();
    resample(C.RESAMPLE_DENSITY, C.RESAMPLE_MAX_DIST);
    generateAngles();
    this.angle = angles.get(0).y;
  }

  // Resample the vertices by traversing the curve and store the vertices that exceed the given segment distance
  void resample(float samplingRate, float segmentDist) {    
    ArrayList<PVector> newVertices = new ArrayList<PVector>();
    newVertices.add(vertices.get(0));

    for (float t=0; t<1; t+=samplingRate) {
      //int floatingIndex = map(t, 0, 1, 0, gcode.vertices.size()-1);
      PVector lerpVert = getInterpolatedVertex(t);
      if (PVector.dist(newVertices.get(newVertices.size()-1), lerpVert) > segmentDist)
        newVertices.add(lerpVert);
    }

    vertices = newVertices;
    getBBox();
  }

  // Calculate angles and angle derivatives from vertices
  void generateAngles() {

    angles.clear();
    dAngles.clear();

    // for each vertes
    for (int iVertex=0; iVertex<vertices.size(); iVertex++) {     
      PVector vertex = vertices.get(iVertex); 

      // Determine sides of the triangle stepperArm, stepperDisc, vertexToDraw
      float a = vertex.mag(); // Being the distance from vertex to center
      float b = arm.radius;
      float c = PVector.sub(disc.center, arm.center).mag();
      // Determine the angle of the coordinate wrt horizontal
      float coordinateAngle = atan2(vertex.y, vertex.x); 

      // calculate all the angles of the triangle using the rule of cosines
      float A = acos((b*b + c*c - a*a)/(2*b*c));
      float B = acos((c*c + a*a - b*b)/(2*c*a));
      float C = acos((a*a + b*b - c*c)/(2*a*b));

      // Transform the coordinate angle to match the calculated angle
      PVector thisAngle = new PVector(A, PI - C -coordinateAngle, vertex.z);  

      // Only add derivative if there are more than 2 angles calculated
      PVector dAngle = (angles.size()-1 >= 0) ?  
        PVector.sub(thisAngle, angles.get(angles.size()-1))  : new PVector() ;

      // Ugly solution to the derivative being too large if the angle goes from 2PI to 0
      if (dAngle.x > PI) dAngle.x = 0;
      if (dAngle.y > PI) dAngle.y = 0;
      if (dAngle.x < -PI) dAngle.x = 0;
      if (dAngle.y < -PI) dAngle.y = 0;

      // Transfer the z position (penup-pendown) to the angle and derivative
      thisAngle.z = vertex.z > 0 ? 1.0 : 0.0;
      dAngle.z = vertex.z > 0 ? 1.0 : 0.0;

      // Add the angle pair
      angles.add(thisAngle);
      // Add the derivative pair
      dAngles.add(dAngle);
    }

    // Add the last derivative to be the difference between the last angle and the first angle
    dAngles.add(PVector.sub(angles.get(angles.size()-1), angles.get(0)));
  }

  // Draw the toolpath
  void draw() {
    noFill();
    pushMatrix();
    {
      rotate(angle);

      // Draw bounding box
      stroke(0, 255, 0 );
      beginShape();
      {
        vertex(bbMin.x, bbMin.y);
        vertex(bbMin.x, bbMax.y);
        vertex(bbMax.x, bbMax.y);
        vertex(bbMax.x, bbMin.y);
        vertex(bbMin.x, bbMin.y);
      }
      endShape();

      // draw lines
      strokeWeight(1);
      beginShape();
      {
        for (PVector v : vertices) {

          // draw pen-up in red
          if (v.z > 0)
            stroke(255, 0, 0);
          else
            stroke(100, 100, 100);

          vertex(v.x, v.y);
        }
      }
      endShape();

      // draw vertices
      stroke(0, 0, 255);      
      beginShape(POINTS);
      {
        for (PVector v : vertices) {
          vertex(v.x, v.y);
        }
      }
      endShape();

      // Draw the center
      fill(255, 0, 0);
      circle(center.x, center.y, 10);

      // Draw the cursor
      PVector activeVector = vertices.get(iCursor); 
      circle(activeVector.x, activeVector.y, 5);
    }
    popMatrix();
  }

  void drawGraphs() {
    // draw the graphs of the angles
    noStroke();
    for (int iAngle=0; iAngle<angles.size(); iAngle++) {
      float x = map(iAngle, 0, angles.size(), 0, width);
      float y = map(angles.get(iAngle).x, -2*PI, 2*PI, C.GRAPH_HEIGHT-C.GRAPH_SCALE*3, C.GRAPH_HEIGHT+C.GRAPH_SCALE*3);
      float y2= map(angles.get(iAngle).y, -2*PI, 2*PI, C.GRAPH_HEIGHT-C.GRAPH_SCALE, C.GRAPH_HEIGHT+C.GRAPH_SCALE);
      fill(255, 0, 0);
      circle(x, y, 2);
      fill(0, 255, 0);
      circle(x, y2, 2);
    }

    // draw the curson on the graphs
    float x = map(iCursor, 0, angles.size(), 0, width);
    float y = map(angles.get(iCursor).x, -2*PI, 2*PI, C.GRAPH_HEIGHT-C.GRAPH_SCALE*3, C.GRAPH_HEIGHT+C.GRAPH_SCALE*3);    
    float y2 = map(angles.get(iCursor).y, -2*PI, 2*PI, C.GRAPH_HEIGHT-C.GRAPH_SCALE, C.GRAPH_HEIGHT+C.GRAPH_SCALE);
    fill(255, 0, 0);
    circle(x, y, 20);
    fill(0, 255, 0);
    circle(x, y2, 20);
  }
}
