import processing.serial.*;

Disc disc;
Arm arm;
GCode gcode;
ToolPath toolPath;

Machine machine;
Serial serialPort;

// Initialize program
void setup() {
  size(800, 800); //<>//
  if (C.SUPPORT_SERIAL) {
    serialPort = new Serial(this, C.SERIAL_PORT, C.SERIAL_BAUDRATE);
    machine = new Machine(serialPort);
  } else {
    machine = new Machine();
  }

  disc = new Disc(C.DISC_CENTER, C.DISC_RADIUS);
  arm = new Arm(C.ARM_CENTER, C.ARM_RADIUS);  
  gcode = new GCode(C.GCODE_FILENAME);  
  toolPath = new ToolPath(gcode.vertices);
  reset();
}

void draw() {
  // Draw everything
  background(100);
  pushMatrix();
  {
    translate(C.DRAW_CENTER.x, C.DRAW_CENTER.y);
    scale(C.DRAW_ZOOM_FACTOR);
    disc.drawTransformed();
    toolPath.draw();
    arm.drawTransformed();
  }
  popMatrix();
  toolPath.drawGraphs();
  // Show what the machine returns
  if (C.VERBOSE && C.SUPPORT_SERIAL) {
    while (serialPort.available() > 0) {
      print(serialPort.readString());
    }
  }
}

// reset drawing
void reset() {
  arm.angle = toolPath.angles.get(0).x;
  disc.angle = toolPath.angles.get(0).y;  
}

// Every time a key is pressed, it's parsed and an action is performed.
// Generally, every inverse action is the same key, with shift.
// 1, turn disc a full revolution, shift-1: reverse
// 2, turn arm a full revolution, shift-2: reverse
// d, set to display mode. Runs the disc motor at a specific rate without stopping. Shift-d to turn off
// s, set strobe on. shift-s to turn it off  
// p, set pen up, shift-p to put it down
// k, kill all. Turn everything off
// h, home. Turn arm in reverse until limit switch is triggered
// m, move. Start drawing.
// t, generate a test pattern (square), shift-T to revert to gcode 

void keyPressed() {
  switch (key) {
  case '1': 
    machine.spinMotor(C.DISC_MOTOR, C.STEPS_PER_REV, C.DEMO_STEPINTERVAL); 
    break;
  case '!': 
    machine.spinMotor(C.DISC_MOTOR, -C.STEPS_PER_REV, C.DEMO_STEPINTERVAL);
    break;
  case '2': 
    machine.spinMotor(C.ARM_MOTOR, 10*C.STEPS_PER_REV, C.DEMO_STEPINTERVAL); 
    break;
  case '@': 
    machine.spinMotor(C.ARM_MOTOR, -10*C.STEPS_PER_REV, C.DEMO_STEPINTERVAL);
    break;
  case 'd': 
    machine.displayModeOn();
    break;
  case 'D': 
    machine.displayModeOff();
    break;
  case 's': 
    machine.strobeOn();
    break;
  case 'S': 
    machine.strobeOff();
    break;
  case 'p': 
    machine.penUp();
    break;
  case 'P': 
    machine.penDown();
    break;
  case 'k': 
    machine.killAll();
    break;
  case 'h': 
    machine.home();
    break;
  case 'm':
    thread("move");
    break;
  case 't':
    ArrayList<PVector> square = new ArrayList<PVector>();    
    square.add(new PVector(-C.DEMO_SQUARE_SIZE, -C.DEMO_SQUARE_SIZE, 0));
    square.add(new PVector(-C.DEMO_SQUARE_SIZE, C.DEMO_SQUARE_SIZE, 0));
    square.add(new PVector(C.DEMO_SQUARE_SIZE, C.DEMO_SQUARE_SIZE, 0));
    square.add(new PVector(C.DEMO_SQUARE_SIZE, -C.DEMO_SQUARE_SIZE, 0));
    square.add(new PVector(-C.DEMO_SQUARE_SIZE, -C.DEMO_SQUARE_SIZE, 0));
    toolPath = new ToolPath(square);
    reset();
    break;
  case 'T':
    toolPath = new ToolPath(gcode.vertices);
    reset();
    break;
  }
}

// This function is called by startiung a thread, so commands ar sent independently from the drawing update
void move() {
  try {
    // go into a form-loop and delay every iteration
    // This is on a separate thread so it doesnt delay the program
    for (int iAngle = 0; iAngle<toolPath.angles.size()-1; iAngle++) {      
      PVector dAngle = toolPath.dAngles.get(iAngle);
      
      // Update arm, disc and toolPath
      arm.angle += dAngle.x;
      disc.angle += dAngle.y;
      toolPath.angle = disc.angle;      
      toolPath.iCursor = iAngle;

      // Calculate amount of steps per motor
      // TODO: calculate interval so both motors take similar time
      int discSteps = (int)(dAngle.y/C.RAD_PER_STEP);
      int armSteps = (int)(dAngle.x/C.RAD_PER_STEP);
      
      // send the commands to the machine
      machine.spinMotor(C.ARM_MOTOR, discSteps, C.ARM_STEP_INTERVAL);
      machine.spinMotor(C.DISC_MOTOR, armSteps, C.DISC_STEP_INTERVAL);
      
      if (dAngle.z > 0) {
        machine.penUp();
        arm.fillColor = color(255,0,0);
      }
      else{
        arm.fillColor = color(255);
        machine.penDown();
      }

      if (C.VERBOSE) { 
        println(String.format("Disc: %d steps, Arm %d steps", discSteps, armSteps));
      }

      // Wait for a bit
      Thread.sleep(C.THREAD_SLEEP_INTERVAL);
    }
  }
  catch (InterruptedException e) {
    println("got interrupted");
  }
}
